#include <avr/io.h>
#include <avr/pgmspace.h>
#include <util/delay.h>
#include "types.h"
#include "lcd.h"


void lcd_init(UINT8 contrast, UINT8 invert)
{
	//init ports
	LCD_PORT |= LCD_RESET_PIN|LCD_CSA_PIN|LCD_CSB_PIN|LCD_RS_PIN|LCD_MOSI_PIN|LCD_SCK_PIN;
	LCD_DDR  |= LCD_RESET_PIN|LCD_CSA_PIN|LCD_CSB_PIN|LCD_RS_PIN|LCD_MOSI_PIN|LCD_SCK_PIN;

	//init spi
	SPCR = (1<<SPE)|(1<<MSTR); 			//enable SPI, Master, clock=fck/4
	SPSR = (1<<SPI2X); 					//double speed -> clock=fck/2

	//lcd reset
	LCD_RESET();
	_delay_ms(10);
	LCD_RESET_OFF();
	_delay_ms(10);

	if((contrast < LCD_CONTRAST_MIN) || (contrast > LCD_CONTRAST_MAX))
	{
		contrast = LCD_CONTRAST_MIN+((LCD_CONTRAST_MAX-LCD_CONTRAST_MIN)/2);
	}
	invert = (invert)?0xA6:0xA7;

	//send lcd init cmds: chip A && B
	lcd_cmd(0xE2, LCD_CHIP_AB);				//reset: 0b11100010
	lcd_cmd(0xAE, LCD_CHIP_AB);				//display: 0b1010111x (off=0, on=1)
	lcd_cmd(0xA0, LCD_CHIP_AB);				//adc direction: normal=0b10100000, reverse=0b10100001
	lcd_cmd(0xC8, LCD_CHIP_AB);				//common output mode select: normal=0b11000---, reverse=0b11001---
	lcd_cmd(invert, LCD_CHIP_AB);			//display: 0b1010011x (normal=0, reverse=1)
	lcd_cmd(0xA4, LCD_CHIP_AB);				//all points: on=0b10100101, off=0b10100100)
	lcd_cmd(0x24, LCD_CHIP_AB);				//voltage regulator: 0b00100xxx (000...111)
	lcd_cmd(0xA2, LCD_CHIP_AB);				//lcd bias: 1/9=0b10100010, 1/7=0b10100011
	lcd_cmd(0x2F, LCD_CHIP_AB);				//power controller: 0b00101xxx (boster|v-regulator|v-follower)
	lcd_cmd(0x81, LCD_CHIP_AB);				//electronic volume mode
	lcd_cmd(contrast, LCD_CHIP_AB);			//0b00101001 //electronic volume register: 0b--xxxxxx (000000...111111)
	lcd_cmd(0x40, LCD_CHIP_AB);				//start line
	lcd_clear();

	lcd_cmd(0xAF, LCD_CHIP_AB);				//display: 0b1010111x (off=0, on=1)

	LCD_CS_OFF();

	return;
}


inline void lcd_spi_write(UINT8 c)
{
	SPDR = c;
	loop_until_bit_is_set(SPSR, SPIF);		//wait for transmission complete

	return;
}


inline void lcd_write(UINT8 c, UINT8 chip)
{
	if(chip & LCD_CHIP_A)
	{
		LCD_CS_A();
	}
	if(chip & LCD_CHIP_B)
	{
		LCD_CS_B();
	}
	
	lcd_spi_write(c);

	LCD_CS_A_OFF();
	LCD_CS_B_OFF();

	return;
}


inline void lcd_data(UINT8 c, UINT8 chip)
{
	LCD_DATA();
	
	lcd_write(c, chip);

	return;
}


inline void lcd_cmd(UINT8 c, UINT8 chip)
{
	LCD_CMD();

	lcd_write(c, chip);

	return;
}


void lcd_clear(void)
{
	UINT8 i, j;

	LCD_CS_A();
	LCD_CS_B();

	for(i=0xB0; i<(0xB0+8); i++)
	{
		LCD_CMD();
		lcd_spi_write(0x10); 				//column msb
		lcd_spi_write(0x0C); 				//column lsb
		lcd_spi_write(i);    				//page

		LCD_DATA();
		for(j=120; j!=0; j--)
		{
			lcd_spi_write(0x00);
		}
	}

	LCD_CS_A_OFF();
	LCD_CS_B_OFF();

	return;
}
